#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (division, print_function, absolute_import,
                        unicode_literals)

from .ktransit import LCModel
from .ktransit import give_me_earth
from .fittransit import FitTransit
from .fittransit import FitTransitWiener
from .fittransit import plot_results



import numpy as np
import kplr
import urllib2

def med_filt(x, y, dt=4.):
    """
    De-trend a light curve using a windowed median.

    """
    x, y = np.atleast_1d(x), np.atleast_1d(y)
    assert len(x) == len(y)
    r = np.empty(len(y))
    for i, t in enumerate(x):
        inds = (x >= t - 0.5 * dt) * (x <= t + 0.5 * dt)
        r[i] = np.median(y[inds])
    return r

def norm_by_quarter(flux,ferr,quarter):
    for i in np.unique(quarter):
        flux[quarter == i] /= np.median(
            flux[quarter == i])
        ferr[quarter == i] /= np.median(
            flux[quarter == i])
    return flux, ferr

def byebyebaddata(flux,ferr,quality):
    finite = np.isfinite(flux)
    qflag = quality == 0
    mask = np.logical_and(finite,qflag)
    flux[~mask] = np.nan
    ferr[~mask] = np.nan
    return flux, ferr

def i_hate_nans(time,flux,ferr):
    finite = np.isfinite(flux)
    return time[finite],flux[finite],ferr[finite]

def get_clean_data(kic,return_extras=False,remove_earth_point=False):
    client = kplr.API()
    star = client.star(kic)
    lcs = star.get_light_curves(short_cadence=False)
    time, flux, ferr, quality,quarter,cadnum = [], [], [], [], [], []
    for lc in lcs:
        try:
            with lc.open() as f:
                # The lightcurve data are in the first FITS HDU.
                hdu_data = f[1].data
                time = np.r_[time,hdu_data["time"]]
                #fluxval = hdu_data["sap_flux"]
                #fluxmed = np.median(fluxval)
                #flux = np.r_[flux,fluxval / fluxmed]
                #ferr = np.r_[ferr,hdu_data["sap_flux_err"] / fluxmed]
                flux = np.r_[flux,hdu_data["sap_flux"]]
                ferr = np.r_[ferr,hdu_data["sap_flux_err"]]
                quality = np.r_[quality,hdu_data["sap_quality"]]
                quarter = np.r_[quarter,f[0].header["QUARTER"] +
                    np.zeros(len(hdu_data["time"]))]
                cadnum = np.r_[cadnum,hdu_data["CADENCENO"]]
        except urllib2.HTTPError:
            continue

    flux, ferr = byebyebaddata(flux,ferr,quality)
    if remove_earth_point:
        flux = remove_post_earth_point(cadnum,flux,quality)
    flux, ferr = norm_by_quarter(flux, ferr,quarter)
    cflux = (flux / med_filt(time,flux,dt=1.0)) - 1.0

    time,cflux,ferr = i_hate_nans(time,cflux,ferr)
    if return_extras:
        extra_d = {'quality': quality,
                    'cadnum': cadnum,
                    'quarter': quarter}
        return (time,cflux,ferr), star, extra_d
    else:
        return (time,cflux,ferr), star,

def remove_post_earth_point(cadnum,flux,quality,window=95):
    ep_idx = [i for i,x in enumerate(
        quality) if x in [1,2,8]]
    for tep in cadnum[ep_idx]:
        trange = np.logical_and(cadnum >= tep,cadnum < tep + window)
        flux[trange] = np.nan
    return flux


from __future__ import division, print_function
import ktransit
from scipy import optimize
import numpy as np



class FitTransit(object):

    def __init__(self):
        self.mod = ktransit.LCModel()
    #    self.nplanets = self.mod.nplanets
        self.free_parameters(fitparstar=[],
        fitparplanet=[])
        self.planetguess_d = {}
        self.uservdata = False

    @property
    def nplanets(self):
        self._nplanets = self.mod.nplanets
        return self._nplanets

    def add_guess_star(self,rho=1.5, ld1=0.2,
        ld2=0.4, ld3=0.0, ld4=0.0, dil=0.0,
        veloffset=0.0, zpt=0.0):
        self.starguess_d={
            'rho': rho,
            'ld1': ld1,
            'ld2': ld2,
            'ld3': ld3,
            'ld4': ld4,
            'dil': dil,
            'veloffset': veloffset,
            'zpt': zpt}
        self.mod.add_star(rho, ld1,
        ld2, ld3, ld4, dil,
        veloffset, zpt)

    def update_star(self,**kwargs):
        goodlist = ['rho', 'ld1',
        'ld2', 'ld3', 'ld4', 'dil',
        'veloffset', 'zpt']
        kwnew = dict([(k,v) for k,v in kwargs.iteritems() if k in goodlist])
        self.mod.update_star(**kwnew)

    def update_planet(self,pnum,**kwargs):
        goodlist = ['T0',
            'period',
            'impact',
            'rprs',
            'ecosw',
            'esinw',
            'rvamp',
            'occ',
            'ell',
            'alb']
        kwnew = dict([(k,v) for k,v in kwargs.iteritems() if k in goodlist])
        self.mod.update_planet(pnum=pnum,**kwnew)

    def add_guess_planet(self,
        T0=1.0, period=1.0, impact=0.1,
        rprs=0.1, ecosw=0.0, esinw=0.0,
        rvamp=0.0, occ=0.0, ell=0.0,
        alb=0.0):
        pname = 'pnum' + str(self.nplanets)
        planet_d = {
            'T0': T0,
            'period': period,
            'impact': impact,
            'rprs': rprs,
            'ecosw': ecosw,
            'esinw': esinw,
            'rvamp': rvamp,
            'occ': occ,
            'ell': ell,
            'alb': alb}
        self.planetguess_d[pname] =  planet_d
        self.mod.add_planet(
            T0=T0, period=period, impact=impact,
            rprs=rprs, ecosw=ecosw, esinw=ecosw,
            rvamp=rvamp, occ=occ, ell=ell,
            alb=alb)

    def add_data(self,time=None, flux=None,
        ferr=None, itime=None,
        ntt=None, tobs=None, omc=None,
        datatype=None):

        if time is None:
            self.time = np.arange(0,10,0.0188)
        else:
            self.time = time
        npt = len(self.time)
        nmax = 1500000

        if flux is None:
            self.flux = np.zeros(npt)
        else:
            self.flux = flux

        if ferr is None:
            self.ferr = np.zeros(npt) + 0.1
        else:
            self.ferr = ferr

        if itime is None:
            default_cadence = 1625.3 / 86400.
            self.itime = np.zeros(npt) + default_cadence
        else:
            self.itime = itime

        if ntt is None:
            self.ntt = np.zeros(self.nplanets)
        else:
            self.ntt = ntt

        if tobs is None:
            self.tobs = np.empty([self.nplanets,nmax])
        else:
            self.tobs = tobs

        if omc is None:
            self.omc = np.empty([self.nplanets,nmax])
        else:
            self.omc = omc

        if datatype is None:
            self.datatype = np.zeros(npt)
        else:
            self.datatype = datatype

        self.mod.add_data(time=time, itime=itime,
            ntt=ntt, tobs=tobs, omc=omc,
            datatype=datatype)

    def add_rv(self,rvtime=None,rvval=None,rverr=None):
        self.uservdata = True
        self.rvtime = rvtime
        self.rvval = rvval
        self.rverr = rverr

        self.mod.add_rv(rvtime=rvtime)


    def free_parameters(self,
        fitparstar=['rho','zpt'],
        fitparplanet=['T0','period','impact','rprs']):
        self.fitparstar = fitparstar
        self.fitparplanet = fitparplanet

    def calc_model(self):
        self._tmod = self.mod.transitmodel
        return self._tmod

    def get_rv_model(self):
        self.rv_model = self.mod._rvmodel
        return self.rv_model

    def residuals(self):
        self._res =  (self.flux - self.calc_model()) / self.ferr

        if self.uservdata:
            self._res = np.r_[
                self._res,(self.rvval - self.get_rv_model()) / self.rverr]

        return self._res


    def ret_lstsq(self,fitpars):
        """

        """
        fitstar_d = dict(zip(self.fitparstar,fitpars[:len(self.fitparstar)]))
        self.update_star(**fitstar_d)

        nps = len(self.fitparstar)
        npp = len(self.fitparplanet)
        for i in xrange(self.nplanets):
            fitplanet_d = dict(
                zip(self.fitparplanet,
                fitpars[nps+i*npp:nps+(i+1)*npp
                ]))

            self.update_planet(pnum=i,**fitplanet_d)

        return self.residuals()

    def do_fit(self):
        fitvalstar = [self.starguess_d[k] for k in self.fitparstar]
        fitvalplanet = []
        for i in xrange(self.nplanets):
            pdic = self.planetguess_d['pnum' + str(i)]
            fitvalplanet = np.r_[fitvalplanet,
            [pdic[k] for k in self.fitparplanet]]
        fitpars = np.r_[fitvalstar,fitvalplanet]
        self.fitout = optimize.leastsq(self.ret_lstsq,fitpars,full_output=True)
        self.make_fitoutdicts()

        self.transitmodel = self.mod.transitmodel


    def make_fitoutdicts(self):
        self.fitresult = self.fitout[0]
        nps = len(self.fitparstar)
        npp = len(self.fitparplanet)
        stellarout = self.fitresult[0:nps]
        self.fitresultstellar = dict(zip(self.fitparstar,stellarout))
        self.fitresultplanets = {}
        for i in xrange(self.nplanets):
            planetout = self.fitresult[nps + i*npp:nps + (i+1)*npp]
            self.fitresultplanets['pnum' + str(i)] = dict(
                zip(self.fitparplanet,planetout))

    def print_results(self):
        print("Best-fitting stellar parameters")
        for k,v in self.fitresultstellar.iteritems():
            print(u'{0}: {1}'.format(k, v))
        print()
        for i in xrange(self.nplanets):
            print("Best-fitting planet parameters for planet {0}".format(i))
            pnum = 'pnum' + str(i)
            for k,v in self.fitresultplanets[pnum].iteritems():
                print(u'{0}: {1}'.format(k, v))
            print()

    def plot_results(self):
        time = self.time
        obsf = self.obsf
        model = self.transitmodel
        fig = plot_results(time,obsf,model)
        return fig

class FitTransitWiener(FitTransit):

    def __init__(self):
        super(FitTransitWiener, self).__init__()
        try:
            from astroML.filters import wiener_filter
            self.wiener_filter = wiener_filter
        except ImportError:
            print('astroLM require to use the Wiener function')
            import sys
            sys.exit(1)

    def residuals(self):
        cm = self.calc_model()
        res =  self.flux - cm

        self._ffilt = self.wiener_filter(self.time, res)

        self._res = (self.flux - (self._ffilt + cm)) / self.ferr

        if self.uservdata:
            self._res = np.r_[
                self._res,(
                    self.rvval - self.get_rv_model()) / self.rverr]

        return self._res







def plot_results(time,obsf,model):
    #import matplotlib
    #matplotlib.use("pdf")
    import matplotlib.pyplot as plt
    fig, (ax1,ax2) = plt.subplots(2, 1, sharex=True, sharey=False)
    ax1.scatter(time,obsf,s=2,color='k',label='obs data')
    ax1.plot(time,model,color='r',label='model fit')
    ax1.legend()
    ax1.set_xlabel('Time (days)')
    ax1.set_ylabel('Norm Flux')
    ax1.set_xlim([np.min(time)-1,np.max(time)+1])
    yrng = np.max(model) - np.min(model)
    ax1.set_ylim([np.min(obsf) - 0.2*yrng,
        np.max(obsf)+0.2*yrng])

    ax2.scatter(time,obsf - model,s=2,color='k',
        label='residuals')
    ax2.legend()
    leg = ax2.legend(loc='best', fancybox=True)
    leg.get_frame().set_alpha(0.5)
    ax2.set_xlabel('Time (days)')
    ax2.set_ylabel('Residuals')
    yrng2 = np.max(obsf - model) - np.min(obsf - model)
    ax2.set_ylim([np.min(obsf - model) - 0.2*yrng2,
        np.max(obsf - model)+0.2*yrng2])


    fig.set_figwidth(10)
    fig.set_figheight(8)

    return fig

def folded_plot(time,obsf,model,period,epoch):
    import matplotlib.pyplot as plt
    raise NotImplementedError




if __name__ == '__main__':
    ##make fake data
    M = ktransit.LCModel()
    M.add_star(rho=5.0)
    M.add_planet(period=1.00,impact=0.1,T0=1.5,rprs=0.1)
    M.add_planet(period=1.20,impact=0.4,T0=0.7,rprs=0.02)
    M.add_data()

    tmod = M.transitmodel
    #addnoise
    from numpy.random import normal
    tmod_noisy = tmod + normal(0.0,0.00001,size=len(tmod))

    #what shall we fit for
    freeparstar = ['rho']
    freeparplanet = [
    'period','T0','impact','rprs']

    #guess some parameters
    fitT = FitTransit()
    fitT.add_guess_star(rho=7.0)
    fitT.add_guess_planet(period=1.001,impact=0.4,T0=1.501,rprs=0.08)
    fitT.add_guess_planet(period=1.201,impact=0.7,T0=0.701,rprs=0.01)
    fitT.add_data(time=M.time,flux=tmod_noisy, ferr=0.00001)
    fitT.free_parameters(freeparstar,freeparplanet)
    fitT.do_fit()

    #update model

    M.update_star(**fitT.fitresultstellar)
    for i in xrange(fitT.nplanets):
        use_d = fitT.fitresultplanets['pnum' + str(i)]
        M.update_planet(pnum=i,**use_d)

    fig = plot_results(M.time,tmod_noisy+1.0,fitT.transitmodel+1.0)
    fig.savefig('fitresult.pdf')




import numpy as np
from _tmodtom import transitmodel

class LCModel(object):

    def __init__(self):
        self.nplanets = 0
        self.T0 = []
        self.period = []
        self.impact = []
        self.rprs = []
        self.ecosw = []
        self.esinw = []
        self.rvamp = []
        self.occ = []
        self.ell = []
        self.alb = []
        self.rvtime = []

    def add_star(self, rho=1.5, ld1=0.2,
        ld2=0.4, ld3=0.0, ld4=0.0, dil=0.0,
        veloffset=0.0, zpt=0.0):
        """
        add details of the star about which the planet(s) orbit
        ldX are the limb darkening parameters
        if only ld1 and and ld2 are non-zero a quadrativ limb darkening
        model is used
        if ld3 and ld4 are also non-zoro then a 4-parameter limb
        darkening law is used

        rho = mean stellar density in g/cc**3
        dil is the proportion of the total light not coming from the
            target star 0.5 means thatyou ahve two stars of equal
            brightness
        """
        self.rho = rho
        self.ldp = [ld1,ld2,ld3,ld4]
        self.dil = dil
        self.veloffset = veloffset
        self.zpt = zpt

    def update_star(self,**kwargs):
        [setattr(self,k,v) for k,v in kwargs.iteritems()]

    def update_planet(self,pnum,**kwargs):
        for k,v in kwargs.iteritems():
            valarr = getattr(self,k)
            valarr[pnum] = v
            setattr(self,k,valarr)


    def add_planet(self,replace=None,
        T0=1.0, period=1.0, impact=0.1,
        rprs=0.1, ecosw=0.0, esinw=0.0,
        rvamp=0.0, occ=0.0, ell=0.0,
        alb=0.0):
        if replace == None:
            self.nplanets = self.nplanets + 1
            pnum = self.nplanets - 1
            self.add_dimention_to_planet_params()
        else:
            pnum = replace

        self.T0[pnum] = T0
        self.period[pnum] = period
        self.impact[pnum] = impact
        self.rprs[pnum] = rprs
        self.ecosw[pnum] = ecosw
        self.esinw[pnum] = esinw
        self.rvamp[pnum] = rvamp
        self.occ[pnum] = occ
        self.ell[pnum] = ell
        self.alb[pnum] = alb

    def add_dimention_to_planet_params(self):
        self.T0 = np.r_[self.T0,0.0]
        self.period = np.r_[self.period,0.0]
        self.impact = np.r_[self.impact,0.0]
        self.rprs = np.r_[self.rprs,0.0]
        self.ecosw = np.r_[self.ecosw,0.0]
        self.esinw = np.r_[self.esinw,0.0]
        self.rvamp = np.r_[self.rvamp,0.0]
        self.occ = np.r_[self.occ,0.0]
        self.ell = np.r_[self.ell,0.0]
        self.alb = np.r_[self.alb,0.0]

    def add_data(self, time=None, itime=None,
        ntt=None, tobs=None, omc=None,
        datatype=None):
        """
        Add data after all the planets are added!!
        """
        if time is None:
            self.time = np.arange(0,10,0.0188)
        else:
            self.time = time
        npt = len(self.time)
        nmax = 1500000

        if itime is None:
            default_cadence = 1625.3 / 86400.
            self.itime = np.zeros(npt) + default_cadence
        else:
            self.itime = itime

        if ntt is None:
            self.ntt = np.zeros(self.nplanets)
        else:
            self.ntt = ntt

        if tobs is None:
            self.tobs = np.empty([self.nplanets,nmax])
        else:
            self.tobs = tobs

        if omc is None:
            self.omc = np.empty([self.nplanets,nmax])
        else:
            self.omc = omc

        if datatype is None:
            self.datatype = np.zeros(npt)
        else:
            self.datatype = datatype

    def add_rv(self,rvtime=None, rvitime=None):
        if rvtime is None:
            self.rvtime = np.arange(
                self.T0[0],4.*self.period[0],1.0)
        else:
            self.rvtime = rvtime

        if rvitime is None:
            default_cadence = 30. / 1440. #30 mins
            self.rvitime = np.zeros_like(self.rvtime) + default_cadence
        else:
            self.rvitime = rvitime


    @property
    def transitmodel(self):
        """
        return a transit model
        calling of model is

        transitmodel(nplanets,sol,time,itime,
        ntt,tobs,omc,datatype)

        sol is [rho,ld1,ld2,ld3,ld4,
        dil,veloffset,zpt,T0,per,b,rprs,ecosw,esinw,
        rvamp,occ,ell,alb]
        """
        ld1,ld2,ld3,ld4 = self.ldp
        sol = np.zeros(8 + self.nplanets*10)

        sol[0:8] = [self.rho,ld1,ld2,ld3,ld4,self.dil,
            self.veloffset,self.zpt]

        sol[8:] = np.array([self.T0,self.period,
            self.impact,self.rprs,
            self.ecosw,self.esinw,
            self.rvamp,self.occ,
            self.ell,self.alb]).T.flatten()

        if np.shape(self.rvtime)[0] != 0:
            time = np.r_[self.time,self.rvtime]
            itime = np.r_[self.itime,self.rvitime]
            datatype = np.r_[
                self.datatype,np.ones_like(self.rvtime)]

            fmod = transitmodel(self.nplanets,
                sol,time,itime,self.ntt,
                self.tobs,self.omc,datatype)
            nrv = np.shape(self.rvtime)[0]
            self._transitmodel = fmod[:-nrv]
            self._rvmodel = fmod[-nrv:]

        else:
            time = self.time
            itime = self.itime
            datatype = self.datatype

            self._transitmodel = transitmodel(self.nplanets,
                sol,time,itime,self.ntt,
                self.tobs,self.omc,datatype)
            self._rvmodel = None



        return self._transitmodel - 1.0

    @property
    def rvmodel(self):
        try:
            return self._rvmodel
        except AttributeError:
            tmod = self.transitmodel
            return self._rvmodel



    def get_ancil_vals(self):
        npt = len(self.time)
        itime = np.zeros(self._npt) + (self.cadence)
        ntt = np.zeros(self._nplanets)
        tobs = np.empty([self._nplanets,npt])
        omc = np.empty([self._nplanets,npt])
        datatype = np.zeros(npt)
        return [itime,ntt,tobs,omc,datatype]


def give_me_earth():
    M = LCModel()
    M.add_star()
    M.add_planet(rprs=0.009155,period=365.25)
    M.add_data(time=np.arange(0,1000,0.0188))
    return (M.time, M.transitmodel)


















#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from numpy.distutils.core import setup, Extension


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()


# First, make sure that the f2py interfaces exist.
interface_exists = os.path.exists("ktransit/tmodtom.pyf")
if "interface" in sys.argv or not interface_exists:
    # Generate the Fortran signature/interface.
    cmd = "cd k-transit;"
    cmd += "f2py tmodtom.f -m _tmodtom -h tmodtom.pyf"
    cmd += " --overwrite-signature"
    os.system(cmd)
    if "interface" in sys.argv:
        sys.exit(0)

# Define the Fortran extension.
tmodtom = Extension("ktransit._tmodtom", ["ktransit/tmodtom.pyf", "ktransit/tmodtom.f"])

setup(
    name="ktransit",
    url="https://github.com/mrtommyb/ktransit",
    version="0.2.1",
    author="Tom Barclay",
    author_email="tom@tombarclay.com",
    description="",
    long_description="",
    packages=["ktransit", ],
    ext_modules=[tmodtom, ],
    classifiers=[
        # "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
)
